# adapt d'une interface de Jean-Guillaume Dumas (Jean-Guillaume.Dumas@imag.fr), 
# initialement prvue pour le jeu de dames 

# resterait a :
# 	coder les tests de fin dans l'interface : 3 repet, mat, pat, aux bons endroits
# 	permettre les retours en arriere et le demarrage d'une nouvelle partie

#!/usr/local/bin/wish
set HOME $env(HOME)
### default location of chess executable
set DEFAUTDIR [pwd]
### default executables
set DEFAUTPROG DEFAUTDIR/Nemja
### Architecture dependant execution of chess program
proc Execute {Programme Fichier Temps Coup} {
	return [exec $Programme $Fichier $Temps $Coup]
}

# temps par defaut
set DEFAUTTIME 1
# joueur en cours
set JOUEUR -1
# couleur en cours
set COLOR 1
# dernier coup joue (cases + flags roque)
set LASTMOVE ""
# coup courant
set CURRMOVE ""
# coup a afficher
set STRMOVE ""
# coordonnees du 1er click dans le cas d'un coup
set PREVIOUSI -1
set PREVIOUSJ -1
set prevCoordI -1
set prevCoordJ -1
# ce qu'il y avait sur la case d'arrivee dans le cas d'un deplacement
set PREVIOUSC 0
# variable indiquant l'action en cours (analyse, joue..)
set COMPUTING ""
# repertoire courant
set LOCAL [file dirname $argv0]
# indique s'il s'agit d'un match humain vs. humain, humain vs. ordi ou ordi vs. ordi
set ORDI 0
# indique si l'ordinateur est en train de jouer
set COMPUTERMOVE 0

# fichiers dans lesquels on stockera les positions
set FILE(1) ""
set FILE(-1) ""
# temps globaux des blancs et des noirs
set GLOBALTIME(-1) 0
set GLOBALTIME(1) 0
# retard global
set BIGLATE(1) 0
set BIGLATE(-1) 0
# dernier retard
set GLOBALLAST(-1) 0
set GLOBALLAST(1) 0
# nombre de coups effectues par chaque camp
set NBCOUPS(-1) 0
set NBCOUPS(1) 0
# coordonnes passees en parametre
set coordI -1
set coordJ -1

# indique la couleur dont on est en train de s'occuper
set CHANGINCOLOR 0
# variable indiquant si c'est game over
set OVER 0
# Image size
set IMS    56
set EXTRA 20
# Plateau dimension : 8x8 with 2 rows occupied by each player
set DIM    8
set RAG    2
# materiel present sur l'echiquier
set WHITES 39
set BLACKS 39
# retards accumules
set RETARD(1) 0
set RETARD(-1) 0
# regle des 50 coups
set CINQUANTE 0

# flags sur le roque
set PRB 1
set PRN 1
set GRB 1
set GRN 1

# Absolute value
proc ABS {a} {
	return [expr ($a <0) ? -$a : $a]
}

## Convert integer to char
proc charCol {x} {
	return [string range "abcdefgh" $x $x]
}

## Convert char to integer
proc intCol {c} {
	return [string first $c "abcdefgh"]
}

##########################################
# Initialisations et definitions diverses
##########################################

# definit l'emplacement du principal executable (numerote car il peut y en avoir 2)
proc SetProg {num program time} {
	global env ORDI PROG ARGS MAXTIME FILE PLATEAU DEFAUTDIR 
	set prog [file join $DEFAUTDIR [file tail $program]]
	if {! [file executable $prog]} {
		set prog $program
	}
	set PROG($num) $prog
	if {! [file executable $PROG($num)]} {
		puts "Erreur : $PROG($num) n'est pas executable."
		exit
	}
	set ARGS($num) $time

	set t1 [clock clicks]
	after 1000
	set t2 [clock clicks]
	set MAXTIME($num) [expr ([ABS $ARGS($num)]*10+1)*($t2 - $t1)/10]

	set FILE($num) Temp/xch_f$num.jgd
	set PLATEAU [file join $DEFAUTDIR Temp/xch_bo.jgd]

	file delete -force $FILE($num)
	puts "Coups joues par $PROG($num) $ARGS($num) dans \t: $FILE($num)"
    
	set ORDI [expr $ORDI - 1]
}

# on lit les arguments en ligne de commande pour connaitre les caracteristiques de chaque programme
if {[expr [llength $argv] > 0]} {
	# One prog    
	SetProg 1 [lindex $argv 0] [lindex $argv 1]
	set PREVIOUSC -1
}
if {[expr [llength $argv] > 2]} {
	# Two progs
	SetProg -1 [lindex $argv 2] [lindex $argv 3]
}

# affiche une boite de dialogue donnant le choix entre un jeu humain ou ordinateur
proc Chooser {color} {
	global CHANGINCOLOR
	set CHANGINCOLOR $color
	toplevel .joueurs
	if {[expr $color == 1]} {
		set textcolor Blancs
	} else {
		set textcolor Noirs
	}
	label .joueurs.lab -text [concat "Les " $textcolor " sont joues par :"] -width 30
	frame .joueurs.ser
	button .joueurs.ser.u -text Mulot -command ChooserUser -width 10
	button .joueurs.ser.p -text Programme -command { ChooserProg $CHANGINCOLOR } -width 10
	pack .joueurs.ser.p .joueurs.ser.u -side left -padx 5
	pack .joueurs.lab .joueurs.ser -side top -pady 5
}

# procedure appelee dans le cas d'un click sur "programme" a l'issue de la procedure Chooser
proc ChooserProg {textcolor} {
	global DEFAUTPROG DEFAUTTIME CHANGINCOLOR
	if {[expr $CHANGINCOLOR == 1]} {
		set textcolor Blancs
	} else {
		set textcolor Noirs
	}
	destroy .joueurs
	toplevel .programme
	label .programme.lab -text [concat "Temps de calcul pour les " $textcolor " :"] -width 30
	frame .programme.mid
	entry .programme.mid.t -width 20 -bg white -relief sunken -textvariable midt
	label .programme.mid.r -text secondes -width 10
	frame .programme.bot
	button .programme.bot.b -text "OK" -command { SetChoice $DEFAUTPROG $midt }
	button .programme.bot.d -text "Dfaut" -command { SetChoice $DEFAUTPROG $DEFAUTTIME }
	pack .programme.mid.t .programme.mid.r -side left
	pack .programme.bot.b .programme.bot.d -side right -padx 4
	pack .programme.lab .programme.mid .programme.bot -side top -pady 4
}

# procedure appelee dans le cas d'un click sur "mulot" a l'issue de la procedure Chooser
proc ChooserUser {} {
	global CHANGINCOLOR ORDI ARGS
	destroy .joueurs
	if {[expr $ORDI == -1]} {
		set actualone [expr $ARGS(1) / [ABS $ARGS(1)]]
		if {[expr $actualone == $CHANGINCOLOR]} {
			set ORDI 0
		}
	} else {
		if {[expr $ORDI == -2]} {
			set actualone [expr $ARGS(1) / [ABS $ARGS(1)]]
			if {[expr $actualone == $CHANGINCOLOR]} {
				global FILE PROG ARGS MAXTIME
				set FILE(1) $FILE(-1)
				set PROG(1) $PROG(-1)
				set ARGS(1) $ARGS(-1)
				set MAXTIME(1) $MAXTIME(-1)
			} 
			set ORDI -1
		}
	}
	destroy .plateau
	BuildTable
}

# choix du programme et du temps alloue
proc SetChoice {program time} {
	global CHANGINCOLOR ORDI ARGS
	destroy .programme
	set numprog 1
	if {[expr $ORDI < 0]} {
		set actualone [expr $ARGS(1) / [ABS $ARGS(1)]]
		if {[expr $actualone == $CHANGINCOLOR]} {
			set ORDI [expr $ORDI + 1]
			set numprog 1
		} else {
			set numprog -1
			if {[expr $ORDI < -1]} {
				set ORDI [expr $ORDI + 1]
			}
		}
	}
	SetProg $numprog $program [expr $CHANGINCOLOR * [ABS $time]]
	destroy .plateau
	BuildTable
}   

### Initial Position
set lignes(7) "-4 -2 -3 -5 -6 -3 -2 -4"
set lignes(6) "-1 -1 -1 -1 -1 -1 -1 -1"
set lignes(5) "0 0 0 0 0 0 0 0"
set lignes(4) "0 0 0 0 0 0 0 0"
set lignes(3) "0 0 0 0 0 0 0 0"
set lignes(2) "0 0 0 0 0 0 0 0"
set lignes(1) "1 1 1 1 1 1 1 1"
set lignes(0) "4 2 3 5 6 3 2 4"

# Pictures
image create photo CaseVide  -file $LOCAL/Images/empty.gif
image create photo pionB -file $LOCAL/Images/wp.gif
image create photo pionN -file $LOCAL/Images/bp.gif
image create photo cavalB -file $LOCAL/Images/wn.gif
image create photo cavalN -file $LOCAL/Images/bn.gif
image create photo fouB -file $LOCAL/Images/wb.gif
image create photo fouN -file $LOCAL/Images/bb.gif
image create photo tourB -file $LOCAL/Images/wr.gif
image create photo tourN -file $LOCAL/Images/br.gif
image create photo dameB -file $LOCAL/Images/wq.gif
image create photo dameN -file $LOCAL/Images/bq.gif
image create photo roiB -file $LOCAL/Images/wk.gif
image create photo roiN -file $LOCAL/Images/bk.gif
set OBJET "roiN dameN tourN fouN cavalN pionN CaseVide pionB cavalB fouB tourB dameB roiB"

# Colors
proc GeneralBG {} { return \#0000A0 }
proc ButtonBG {} { return \#0000A0 }
proc EmptyBG {} { return \#A0A0FF }
proc PressedBG {} { return \#FF0000 }
proc PlateauBG {} { return \#007070 }
proc MButtonBG {} { return \#FFFF00 }
proc MButtonHBG {} { return \#FFFF00 }
proc MButtonABG {} { return \#FFA000 }

# dessine l'echiquier initial + cases
proc BuildBoard {} {
	global IMS
	global DIM
	global EXTRA
	for {set i 0} {[expr $i < $DIM]} {incr i} {
		frame .plateau.r$i -height $IMS -width [expr $IMS * $DIM] -background [PlateauBG] -highlightbackground [PlateauBG] 
		canvas .plateau.r$i.canel -height $EXTRA -width $EXTRA -background [PlateauBG] -highlightbackground [PlateauBG] 
		pack .plateau.r$i.canel -side left
		.plateau.r$i.canel create text 13 12 -text $i -fill white
		for {set j 0} {[expr $j < $DIM]} {incr j} {
			if {($i+$j)%2} { 
				button .plateau.r$i.b$i$j -anchor e -image [sState [nState $i $j] ] -activebackground [EmptyBG] -highlightbackground black -background [EmptyBG] -borderwidth 0 -command "if \{\[Select $i $j\]\} \{ Next \}"
				pack .plateau.r$i.b$i$j -side left
			} else { 
				button .plateau.r$i.b$i$j -anchor e -image [sState [nState $i $j] ] -activebackground [ButtonBG] -highlightbackground black -background [ButtonBG] -borderwidth 0 -command "if \{\[Select $i $j\]\} \{ Next \}"
				pack .plateau.r$i.b$i$j -side left
			}	       
		}
		canvas .plateau.r$i.caner -height $EXTRA -width $EXTRA -background [PlateauBG] -highlightbackground [PlateauBG] 
		pack .plateau.r$i.caner -side left
		.plateau.r$i.caner create text 9 12 -text $i -fill white
		pack .plateau.r$i -side bottom
	}    
}

## Packs start and exit buttons
proc BuildTable {} {
	global DIM
	global IMS
	global EXTRA
	frame .plateau -background [PlateauBG]
	frame  .plateau.frame -height $IMS -width [expr ($IMS * $DIM)]  -background [PlateauBG]
	for {set i 0} {[expr $i < $DIM]} {incr i} {
		set let [charCol $i]
		canvas .plateau.frame.can$let -height $EXTRA -width $IMS  -background [PlateauBG] -highlightbackground [PlateauBG] 
		pack .plateau.frame.can$let -side left
		.plateau.frame.can$let create text 19 9 -text $let -fill white
	}
	pack .plateau.frame -side bottom

	BuildBoard
	frame  .plateau.frameb -height $IMS -width [expr $IMS * $DIM] -background [PlateauBG] -highlightbackground [PlateauBG]
	for {set i 0} {[expr $i < $DIM]} {incr i} {
		set let [charCol $i]
		canvas .plateau.frameb.can$let -height 25 -width $IMS -background [PlateauBG] -highlightbackground [PlateauBG]
		pack .plateau.frameb.can$let -side left
		.plateau.frameb.can$let create text 19 17 -text $let -fill white
	}
	pack .plateau.frameb -side bottom

	frame  .plateau.diff -height 1 -width [expr $IMS * $DIM]  -background [PlateauBG]
	frame  .plateau.diff.fr -height 10 -width 40  -background [PlateauBG]
	button  .plateau.diff.bla -text " Blancs " -background white -foreground black -width 9 -relief flat -command {Chooser 1}
	button  .plateau.diff.noi -text "  Noirs " -background black -foreground white -width 9  -relief flat -command {Chooser -1}
	label  .plateau.diff.ncs -textvariable WHITES -background white -foreground black -width 3 -borderwidth 7
	label  .plateau.diff.rs -textvariable BLACKS -background black -foreground white -width 3 -borderwidth 7

	global ORDI
	if {$ORDI} {
		label  .plateau.diff.blaret -text " retards" -background white -foreground black -width 8 -borderwidth 7
		label  .plateau.diff.noiret -text " retards" -background black -foreground white -width 8 -borderwidth 7
		label  .plateau.diff.ncsret -textvariable RETARD(1) -background white -foreground black -width 4 -borderwidth 7
		label  .plateau.diff.rsret -textvariable RETARD(-1) -background black -foreground white -width 4 -borderwidth 7
		if {[expr $ORDI == -2]} {    
			pack .plateau.diff.noiret .plateau.diff.rsret .plateau.diff.noi .plateau.diff.rs .plateau.diff.fr -side right -padx 2
			pack .plateau.diff.ncs .plateau.diff.bla .plateau.diff.ncsret .plateau.diff.blaret -side left -padx 2

			frame  .plateau.time -height 1 -width [expr $IMS * $DIM]  -background [PlateauBG]
			frame  .plateau.time.fr -height 10 -width 40  -background [PlateauBG]
			label  .plateau.time.ncs -text " Temps global : " -background white -foreground black -width 16
			label  .plateau.time.rs -text " Temps global : " -background black -foreground white -width 16
			label  .plateau.time.bla -textvariable GLOBALTIME(1) -background white -foreground black -width 10
			label  .plateau.time.noi -textvariable GLOBALTIME(-1) -background black -foreground white -width 10
			pack .plateau.time.noi .plateau.time.rs .plateau.time.fr -side right
			pack .plateau.time.ncs .plateau.time.bla -side left
			pack .plateau.time -side bottom

			frame  .plateau.last -height 1 -width [expr $IMS * $DIM]  -background [PlateauBG]
			frame  .plateau.last.fr -height 10 -width 40  -background [PlateauBG]
			label  .plateau.last.ncs -text "Dernier temps : " -background white -foreground black -width 16
			label  .plateau.last.rs -text "Dernier temps : " -background black -foreground white -width 16
			label  .plateau.last.bla -textvariable GLOBALLAST(1) -background white -foreground black -width 10
			label  .plateau.last.noi -textvariable GLOBALLAST(-1) -background black -foreground white -width 10
			pack .plateau.last.noi .plateau.last.rs .plateau.last.fr -side right
			pack .plateau.last.ncs .plateau.last.bla -side left
			pack .plateau.last -side bottom
		} else {
			global ARGS
			.plateau.diff.fr configure -width 200
			if {[expr $ARGS(1)>0]} {
				## Whites
				pack .plateau.diff.noi .plateau.diff.rs .plateau.diff.fr -side right -padx 2
				pack .plateau.diff.ncs .plateau.diff.bla .plateau.diff.ncsret .plateau.diff.blaret -side left -padx 2
			} else {
				## blacks
				.plateau.diff.rsret configure -textvariable RETARD(1)
				pack .plateau.diff.noiret .plateau.diff.rsret .plateau.diff.noi .plateau.diff.rs .plateau.diff.fr -side right -padx 2
				pack .plateau.diff.ncs .plateau.diff.bla -side left -padx 2
			}
		}
	} else {
		.plateau.diff.fr configure -width 280
		pack .plateau.diff.noi .plateau.diff.rs .plateau.diff.fr -side right  -padx 2
		pack .plateau.diff.ncs .plateau.diff.bla -side left -padx 2
	}
	
	pack .plateau.diff -side bottom
	button .plateau.exit -text "Exit" -command { Quit "Exit"; exit } -background [MButtonBG] -highlightbackground [MButtonHBG] -activebackground [MButtonABG] 
	button .plateau.start -text "Start" -command {Start} -background [MButtonBG] -highlightbackground [MButtonHBG] -activebackground [MButtonABG] 
	label  .plateau.text -text "Dernier coup joue : " -foreground white -background [PlateauBG] 
	label  .plateau.coup -textvariable STRMOVE -foreground red -background [PlateauBG] 
	label  .plateau.comp -textvariable COMPUTING -foreground red -background [PlateauBG] -relief sunken -width 11
	pack .plateau.exit .plateau.comp -side right -padx 10 -pady 10 
	pack .plateau.start -side left -padx 10 -pady 10
	pack .plateau.text .plateau.coup -side left
	pack .plateau
}

# initialise le jeu en construisant la fenetre principale
proc Init {} {
	BuildTable
}

# demarre en "appelant" le joueur suivant
proc Start {} {
	global OVER; set OVER 0
	Next
}

###########################
# Procedures utiles au jeu
###########################

# ecrit la position courante dans un fichier, pour une couleur
proc DisplayGame {JOUEUR} {
	global PLATEAU
	global lignes
	set FILEID($JOUEUR) [open $PLATEAU w 0600]
	for {set i 7} {[expr $i >= 0]} {incr i -1} {
		set clignes ""
		for {set j 0} {[expr $j < 8]} {incr j} {
			set val [lindex $lignes($i) $j]
			switch -- $val {
				-1 {append clignes p}
				-2 {append clignes c}
				-3 {append clignes f}
				-4 {append clignes t}
				-5 {append clignes d}
				-6 {append clignes r}
				1 {append clignes P}
				2 {append clignes C}
				3 {append clignes F}
				4 {append clignes T}
				5 {append clignes D}
				6 {append clignes R}
				0 {append clignes " "}
			}
		}
		puts $FILEID($JOUEUR) $clignes
	}
	close $FILEID($JOUEUR)
}

# Square state : pawn, knight, rook.. (with color), or empty
proc nState {i j} {
	global lignes
	return  [lindex $lignes($i) [expr $j] ]  
}

# Associate picture to piece
proc sState {type} {
	global OBJET
	return [lindex $OBJET [expr $type+6 ] ]
}

# affiche une boite de dialogue pour le choix de la piece promue en i,j
proc Promotion {} { 
	global COLOR
	global PREVIOUSI
	global PREVIOUSJ
	global prevCoordI
	global prevCoordJ
	
	set prevCoordI $PREVIOUSI
	set prevCoordJ $PREVIOUSJ

	toplevel .promotion
	frame .promotion.ser
	label .promotion.lab -text "Quelle piece voulez-vous ?" -width 30 
	button .promotion.ser.a -text Dame -command { destroy .promotion ; TerminePromotion [expr $COLOR * 5] } -width 10 
	button .promotion.ser.b -text Tour -command { destroy .promotion ; TerminePromotion [expr $COLOR * 4] } -width 10
	button .promotion.ser.c -text Fou -command { destroy .promotion ; TerminePromotion [expr $COLOR * 3] } -width 10
	button .promotion.ser.d -text Cavalier -command { destroy .promotion ; TerminePromotion [expr $COLOR * 2] } -width 10
	pack .promotion.ser.a .promotion.ser.b .promotion.ser.c .promotion.ser.d -side left -padx 5
	pack .promotion.lab .promotion.ser -side top -pady 5 
}

proc TerminePromotion {piece} {
	global WHITES
	global BLACKS
	global COLOR
	global coordI
	global coordJ
	global PREVIOUSI
	global PREVIOUSJ
	global prevCoordI
	global prevCoordJ
	global PRB PRN GRB GRN
	global lignes
	set coup [nState $coordI $coordJ]
	set PREVIOUSI $prevCoordI
	set PREVIOUSJ $prevCoordJ
	
	# mise a jour du compteur de materiel
	if {[expr ($piece == 5)]} {incr WHITES 8}
	if {[expr ($piece == 4)]} {incr WHITES 4}
	if {[expr ($piece == 3)]} {incr WHITES 2}
	if {[expr ($piece == 2)]} {incr WHITES 2}
	if {[expr ($piece == -5)]} {incr BLACKS 8}
	if {[expr ($piece == -4)]} {incr BLACKS 4}
	if {[expr ($piece == -3)]} {incr BLACKS 2}
	if {[expr ($piece == -2)]} {incr BLACKS 2}
	
	## modify the data structure
	set lignes($coordI) [lreplace $lignes($coordI) [expr $coordJ] [expr $coordJ] $piece]
	set lignes($prevCoordI) [lreplace $lignes($prevCoordI) [expr $prevCoordJ] [expr $prevCoordJ] 0]
	
	## display the new configuration
	if {($coordI+$coordJ)%2} { 
		.plateau.r$coordI.b$coordI$coordJ configure -image [sState $piece] -background [EmptyBG]
	} else {
		.plateau.r$coordI.b$coordI$coordJ configure -image [sState $piece] -background [ButtonBG] 
	}
	if {($PREVIOUSI+$PREVIOUSJ)%2} { 
		.plateau.r$PREVIOUSI.b$PREVIOUSI$PREVIOUSJ configure -image [sState 0] -background [EmptyBG]
	} else {
		.plateau.r$PREVIOUSI.b$PREVIOUSI$PREVIOUSJ configure -image [sState 0] -background [ButtonBG] 
	}
	
	## update globals
	set LASTMOVE "[charCol $PREVIOUSJ]$PREVIOUSI[charCol $coordJ]$coordI$PRB$PRN$GRB$GRN" 
	set STRMOVE "[charCol $PREVIOUSJ][expr $PREVIOUSI+1][charCol $coordJ][expr $coordI+1]"
	set PREVIOUSC $coup
	set prevCoordI -1
	set PREVIOUSI -1
	set PREVIOUSJ -1
	set COLOR [expr -$COLOR]	
	return true
}
	

# indique si une case est en echec par la couleur -COLOR
proc UnderCheck {i j} {
	global COLOR
	for {set k 0} { [expr $k < 8] } {incr k} {
		for {set l 0} { [expr $l < 8] } {incr l} {
			set etat [nState $k $l]
			set emp [expr $etat * $COLOR]
			if {$emp < 0} {
				## possibilite de mise en echec
				if {$emp == -1} {
					## pion
					if {$COLOR<0 && [expr [expr $i-1] == $k] && [expr [ABS [expr $j-$l]] == 1]} {return true}
					if {$COLOR>0 && [expr [expr $i+1] == $k] && [expr [ABS [expr $j-$l]] == 1]} {return true}
				}
				if {$emp == -2} {
					## cavalier
					if {[expr [expr $i-2] == $k] && [expr [ABS [expr $j-$l]] == 1]} {return true}
					if {[expr [expr $i+2] == $k] && [expr [ABS [expr $j-$l]] == 1]} {return true}
					if {[expr [expr $j-2] == $l] && [expr [ABS [expr $i-$k]] == 1]} {return true}
					if {[expr [expr $j+2] == $l] && [expr [ABS [expr $i-$k]] == 1]} {return true}
				}
				if {$emp == -3} {
					## fou
					if {[expr $k - $i] == [expr $l - $j]} { 
						if {$i < $k} {
							set caseInitx [expr $i + 1]
							set caseFinx [expr $k - 1]
							set caseInity [expr $j + 1]
						} else {
							set caseInitx [expr $k + 1]
							set caseFinx [expr $i - 1]
							set caseInity [expr $l + 1]
						}	
						set x $caseInitx
						set y $caseInity
						for {} { [expr $x <= $caseFinx ] } {incr x ; incr y} {
							if {[nState $x $y] != 0} {break}
						}
						if {$x > $caseFinx} {return true}
					}
					if {[expr $k - $i] == -[expr $l - $j]} { 
						if {$i < $k} {
							set caseInitx [expr $k - 1]
							set caseFinx [expr $i + 1]
							set caseInity [expr $l + 1]
						} else {
							set caseInitx [expr $i - 1]
							set caseFinx [expr $k + 1]
							set caseInity [expr $j + 1]
						}	
						set x $caseInitx
						set y $caseInity
						for {} { [expr $x >= $caseFinx ] } {incr x -1 ; incr y} {
							if {[nState $x $y] != 0} {break}
						}
						if {$x < $caseFinx} {return true}
					}
				}
				if {$emp == -4} {
					## tour
					if {$i == $k} {
						if {$j < $l} {
							set caseInit [expr $j + 1]
							set caseFin [expr $l - 1]
						} else {
							set caseInit [expr $l + 1]
							set caseFin [expr $j - 1]
						}	
						set x $caseInit
						for {} { [expr $x <= $caseFin ] } {incr x} {
							if {[nState $i $x] != 0} {break}
						}
						if {$x > $caseFin} {return true}
					}
					if {$j == $l} {
						if {$i < $k} {
							set caseInit [expr $i + 1]
							set caseFin [expr $k - 1]
						} else {
							set caseInit [expr $k + 1]
							set caseFin [expr $i - 1]
						}	
						set x $caseInit
						for {} { [expr $x <= $caseFin ] } {incr x} {
							if {[nState $x $j] != 0} {break}
						}
						if {$x > $caseFin} {return true}
					}
				}
				if {$emp == -5} {
					## dame
					if {[expr $k - $i] == [expr $l - $j]} {
						if {$i < $k} {
							set caseInitx [expr $i + 1]
							set caseFinx [expr $k - 1]
							set caseInity [expr $j + 1]
						} else {
							set caseInitx [expr $k + 1]
							set caseFinx [expr $i - 1]
							set caseInity [expr $l + 1]
						}	
						set x $caseInitx
						set y $caseInity
						for {} { [expr $x <= $caseFinx ] } {incr x ; incr y} {
							if {[nState $x $y] != 0} {break}
						}
						if {$x > $caseFinx} {return true}
					}
					if {[expr $k - $i] == -[expr $l - $j]} { 
						if {$i < $k} { 
							set caseInitx [expr $k - 1]
							set caseFinx [expr $i + 1]
							set caseInity [expr $l + 1]
						} else {
							set caseInitx [expr $i - 1]
							set caseFinx [expr $k + 1]
							set caseInity [expr $j + 1]
						}	
						set x $caseInitx
						set y $caseInity
						for {} { [expr $x >= $caseFinx ] } {incr x -1 ; incr y} { 
							if {[nState $x $y] != 0} {break}
						}
						if {$x < $caseFinx} {return true}
					}
					if {$i == $k} {
						if {$j < $l} {
							set caseInit [expr $j + 1]
							set caseFin [expr $l - 1]
						} else {
							set caseInit [expr $l + 1]
							set caseFin [expr $j - 1]
						}	
						set x $caseInit
						for {} { [expr $x <= $caseFin ] } {incr x} {
							if {[nState $i $x] != 0} {break}
						}
						if {$x > $caseFin} {return true}
					}
					if {$j == $l} {
						if {$i < $k} {
							set caseInit [expr $i + 1]
							set caseFin [expr $k - 1]
						} else {
							set caseInit [expr $k + 1]
							set caseFin [expr $i - 1]
						}	
						set x $caseInit
						for {} { [expr $x <= $caseFin ] } {incr x} {
							if {[nState $x $j] != 0} {break}
						}
						if {$x > $caseFin} {return true}
					}
				}
				if {$emp == -6} {
					## roi
					if {[expr [ABS [expr $i-$k]] <= 1] && [expr [ABS [expr $j-$l]] <= 1]} {return true}
				}
			}
		}
	}	
	return false
}

# effectue un coup si les clicks definissent un coup valide
proc Select {i j} { 
	global PREVIOUSC
	global PREVIOUSI
	global PREVIOUSJ
	global LASTMOVE
	global CURRMOVE
	global STRMOVE
	global WHITES
	global BLACKS
	global lignes
	global bTrue
	global bFalse
	global coordI
	global coordJ
	global COLOR
	global JOUEUR
	global PRB
	global PRN
	global GRB
	global GRN
	global CINQUANTE

	# teste si il y a un ancien click
	if {[expr $PREVIOUSI < 0] || [expr $PREVIOUSJ < 0]} { 
		### Wrong move
		.plateau.r$i.b$i$j configure -background [PressedBG]
		set PREVIOUSI $i
		set PREVIOUSJ $j
		return false
	}
	set caseDepart [nState $PREVIOUSI $PREVIOUSJ]
	set coup [nState $i $j]
	# teste s'il y avait qqch de la bonne couleur sur l'ancienne case, et si la case d'arrivee est OK	
	if {[expr [expr $caseDepart * $COLOR] <= 0] || [expr [expr $coup * $COLOR] > 0]} {
		### Wrong move
		if {($PREVIOUSI+$PREVIOUSJ)%2} {
			.plateau.r$PREVIOUSI.b$PREVIOUSI$PREVIOUSJ configure -background [EmptyBG]
		} else {
			.plateau.r$PREVIOUSI.b$PREVIOUSI$PREVIOUSJ configure -background [ButtonBG]
		}
		set PREVIOUSI -1
		set PREVIOUSJ -1
		return false
	}
	# teste si roque OK
	## Blancs
	if {[expr $COLOR > 0 ] && [expr $PREVIOUSI == 0] && [expr $PREVIOUSJ ==4] && [expr $i == 0] && [expr $j ==6] && [expr $caseDepart == 6]} {
		set caseTest1 [nState 0 5]
		set caseTest2 [nState 0 6]
		set caseTest3 [nState 0 7]
		# verification des flags + cases + verifications d'echecs
		if {[expr $PRB == 0] || [expr $caseTest1 != 0] || [expr $caseTest2 != 0] || [expr $caseTest3 != 4] || [UnderCheck 0 4] || [UnderCheck 0 5] || [UnderCheck 0 6]} {
			### Wrong move
			.plateau.r0.b04 configure -background [ButtonBG]
			set PREVIOUSI -1
			set PREVIOUSJ -1
			return false
		}
		### Valid move
		## display the new configuration
		.plateau.r0.b06 configure -image [sState 6] -background [ButtonBG] 
		.plateau.r0.b04 configure -image [sState 0] -background [ButtonBG] 
		.plateau.r0.b05 configure -image [sState 4] -background [EmptyBG]
		.plateau.r0.b07 configure -image [sState 0] -background [EmptyBG]
		
		## modify the data structure
		set lignes(0) [lreplace $lignes(0) [expr 6] [expr 6] 6]
		set lignes(0) [lreplace $lignes(0) [expr 4] [expr 4] 0]
		set lignes(0) [lreplace $lignes(0) [expr 5] [expr 5] 4]
		set lignes(0) [lreplace $lignes(0) [expr 7] [expr 7] 0]
		
		## update global variables
		set PRB 0
		set GRB 0
		set LASTMOVE "[charCol 4]0[charCol 6]0$PRB$PRN$GRB$GRN" 
		set STRMOVE "[charCol 4]1[charCol 6]1"
		set PREVIOUSC $coup
		set PREVIOUSI -1
		set PREVIOUSJ -1
		set COLOR -1	
		return true
	}
	if {[expr $COLOR > 0 ] && [expr $PREVIOUSI == 0] && [expr $PREVIOUSJ ==4] && [expr $i == 0] && [expr $j ==2] && [expr $caseDepart == 6]} {
		set caseTest1 [nState 0 3]
		set caseTest2 [nState 0 2]
		set caseTest3 [nState 0 1]
		set caseTest4 [nState 0 0]
		# verification des flags + cases + verifications d'echecs
		if {[expr $GRB == 0] || [expr $caseTest1 != 0] || [expr $caseTest2 != 0] || [expr $caseTest3 != 0] || [expr $caseTest4 != 4] || [UnderCheck 0 4] || [UnderCheck 0 3] || [UnderCheck 0 2]} {
			### Wrong move
			.plateau.r0.b04 configure -background [ButtonBG]
			set PREVIOUSI -1
			set PREVIOUSJ -1
			return false
		}
		### Valid move
		## display the new configuration
		.plateau.r0.b02 configure -image [sState 6] -background [ButtonBG] 
		.plateau.r0.b04 configure -image [sState 0] -background [ButtonBG] 
		.plateau.r0.b03 configure -image [sState 4] -background [EmptyBG]
		.plateau.r0.b00 configure -image [sState 0] -background [ButtonBG]
		
		## modify the data structure
		set lignes(0) [lreplace $lignes(0) [expr 2] [expr 2] 6]
		set lignes(0) [lreplace $lignes(0) [expr 4] [expr 4] 0]
		set lignes(0) [lreplace $lignes(0) [expr 3] [expr 3] 4]
		set lignes(0) [lreplace $lignes(0) [expr 0] [expr 0] 0]
		
		## update global variables
		set PRB 0
		set GRB 0
		set LASTMOVE "[charCol 4]0[charCol 2]0$PRB$PRN$GRB$GRN" 
		set STRMOVE "[charCol 4]1[charCol 2]1"
		set PREVIOUSC $coup
		set PREVIOUSI -1
		set PREVIOUSJ -1
		set COLOR -1	
		return true
	}
	## Noirs
	if {[expr $COLOR < 0 ] && [expr $PREVIOUSI == 7] && [expr $PREVIOUSJ ==4] && [expr $i == 7] && [expr $j ==6] && [expr $caseDepart == -6]} {
		set caseTest1 [nState 7 5]
		set caseTest2 [nState 7 6]
		set caseTest3 [nState 7 7]
		# verification des flags + cases + verifications d'echecs
		if {[expr $PRN == 0] || [expr $caseTest1 != 0] || [expr $caseTest2 != 0] || [expr $caseTest3 != -4] || [UnderCheck 7 4] || [UnderCheck 7 5] || [UnderCheck 7 6]} {
			### Wrong move
			.plateau.r7.b74 configure -background [EmptyBG]
			set PREVIOUSI -1
			set PREVIOUSJ -1
			return false
		}
		### Valid move
		## display the new configuration
		.plateau.r7.b76 configure -image [sState -6] -background [EmptyBG] 
		.plateau.r7.b74 configure -image [sState 0] -background [EmptyBG] 
		.plateau.r7.b75 configure -image [sState -4] -background [ButtonBG]
		.plateau.r7.b77 configure -image [sState 0] -background [ButtonBG]
		
		## modify the data structure
		set lignes(7) [lreplace $lignes(7) [expr 6] [expr 6] -6]
		set lignes(7) [lreplace $lignes(7) [expr 4] [expr 4] 0]
		set lignes(7) [lreplace $lignes(7) [expr 5] [expr 5] -4]
		set lignes(7) [lreplace $lignes(7) [expr 7] [expr 7] 0]
		
		## update global variables
		set PRN 0
		set GRN 0
		set LASTMOVE "[charCol 4]7[charCol 6]7$PRB$PRN$GRB$GRN" 
		set STRMOVE "[charCol 4]8[charCol 6]8"
		set PREVIOUSC $coup
		set PREVIOUSI -1
		set PREVIOUSJ -1
		set COLOR 1	
		return true
	}
	if {[expr $COLOR < 0 ] && [expr $PREVIOUSI == 7] && [expr $PREVIOUSJ ==4] && [expr $i == 7] && [expr $j ==2] && [expr $caseDepart == -6]} {
		set caseTest1 [nState 7 3] 
		set caseTest2 [nState 7 2]
		set caseTest3 [nState 7 1]
		set caseTest4 [nState 7 0]
		# verification des flags + cases + verifications d'echecs
		if {[expr $GRN == 0] || [expr $caseTest1 != 0] || [expr $caseTest2 != 0] || [expr $caseTest3 != 0] || [expr $caseTest4 != -4] || [UnderCheck 7 4] || [UnderCheck 7 3] || [UnderCheck 7 2]} {
			### Wrong move
			.plateau.r7.b74 configure -background [EmptyBG]
			set PREVIOUSI -1
			set PREVIOUSJ -1
			return false
		}
		### Valid move
		## display the new configuration
		.plateau.r7.b72 configure -image [sState -6] -background [EmptyBG] 
		.plateau.r7.b74 configure -image [sState 0] -background [EmptyBG] 
		.plateau.r7.b73 configure -image [sState -4] -background [ButtonBG]
		.plateau.r7.b70 configure -image [sState 0] -background [EmptyBG]
		
		## modify the data structure
		set lignes(7) [lreplace $lignes(7) [expr 2] [expr 2] -6]
		set lignes(7) [lreplace $lignes(7) [expr 4] [expr 4] 0]
		set lignes(7) [lreplace $lignes(7) [expr 3] [expr 3] -4]
		set lignes(7) [lreplace $lignes(7) [expr 0] [expr 0] 0]
		
		## update global variables
		set PRN 0
		set GRN 0
		set LASTMOVE "[charCol 4]7[charCol 2]7$PRB$PRN$GRB$GRN" 
		set STRMOVE "[charCol 4]8[charCol 2]8"
		set PREVIOUSC $coup
		set PREVIOUSI -1
		set PREVIOUSJ -1
		set COLOR 1	
		return true
	}
	# tentative de prise en passant
	## blancs
	if {[expr $COLOR > 0 ] && [expr $caseDepart == 1] && [expr $coup == 0] && [expr [ABS [expr $j - $PREVIOUSJ]] == 1] && [expr $PREVIOUSI == 4] && [expr [expr $i - $PREVIOUSI] == 1]} {
		if {[expr [intCol [string index $LASTMOVE 0]] == $j] && [expr [string index $LASTMOVE 1] == 6] && [expr [intCol [string index $LASTMOVE 2]] == $j] && [expr [string index $LASTMOVE 3] == 4]} {
			## display the new configuration
			if {($i+$j)%2} { 
				.plateau.r5.b5$j configure -image [sState $caseDepart] -background [EmptyBG]
				.plateau.r4.b4$j configure -image [sState 0] -background [ButtonBG]
			} else {
				.plateau.r5.b5$j configure -image [sState $caseDepart] -background [ButtonBG]
				.plateau.r4.b4$j configure -image [sState 0] -background [EmptyBG]
			}
			if {($PREVIOUSI+$PREVIOUSJ)%2} { 
				.plateau.r$PREVIOUSI.b$PREVIOUSI$PREVIOUSJ configure -image [sState 0] -background [EmptyBG]
			} else {
				.plateau.r$PREVIOUSI.b$PREVIOUSI$PREVIOUSJ configure -image [sState 0] -background [ButtonBG] 
			}
			
			## modify the data structure
			set lignes(5) [lreplace $lignes(5) [expr $j] [expr $j] $caseDepart]
			set lignes(4) [lreplace $lignes(4) [expr $j] [expr $j] 0]
			set lignes($PREVIOUSI) [lreplace $lignes($PREVIOUSI) [expr $PREVIOUSJ] [expr $PREVIOUSJ] 0]
	
			# mise a jour du compteur de materiel
			incr BLACKS -1
	
			## update global variables
			set LASTMOVE "[charCol $PREVIOUSJ]$PREVIOUSI[charCol $j]$i$PRB$PRN$GRB$GRN" 
			set STRMOVE "[charCol $PREVIOUSJ][expr $PREVIOUSI+1][charCol $j][expr $i+1]"
			set PREVIOUSC $coup
			set PREVIOUSI -1
			set PREVIOUSJ -1
			set COLOR -1	
			return true
		} else {
			## wrong move
			if {($i+$j)%2} { 
				.plateau.r$i.b$i$j configure -background [EmptyBG]
			} else {
				.plateau.r$i.b$i$j configure -background [ButtonBG]
			}
			set PREVIOUSI -1
			set PREVIOUSJ -1
			return false
		}
	}
	## Noirs
	if {[expr $COLOR < 0 ] && [expr $caseDepart == -1] && [expr $coup == 0] && [expr [ABS [expr $j - $PREVIOUSJ]] == 1] && [expr $PREVIOUSI == 3] && [expr [expr $i - $PREVIOUSI] == -1]} {
		if {[expr [intCol [string index $LASTMOVE 0]] == $j] && [expr [string index $LASTMOVE 1] == 1] && [expr [intCol [string index $LASTMOVE 2]] == $j] && [expr [string index $LASTMOVE 3] == 3]} {
			## display the new configuration
			if {($i+$j)%2} { 
				.plateau.r2.b2$j configure -image [sState $caseDepart] -background [EmptyBG]
				.plateau.r3.b3$j configure -image [sState 0] -background [ButtonBG]
			} else {
				.plateau.r2.b2$j configure -image [sState $caseDepart] -background [ButtonBG]
				.plateau.r3.b3$j configure -image [sState 0] -background [EmptyBG]
			}
			if {($PREVIOUSI+$PREVIOUSJ)%2} { 
				.plateau.r$PREVIOUSI.b$PREVIOUSI$PREVIOUSJ configure -image [sState 0] -background [EmptyBG]
			} else {
				.plateau.r$PREVIOUSI.b$PREVIOUSI$PREVIOUSJ configure -image [sState 0] -background [ButtonBG] 
			}
			
			## modify the data structure
			set lignes(2) [lreplace $lignes(2) [expr $j] [expr $j] $caseDepart]
			set lignes(3) [lreplace $lignes(3) [expr $j] [expr $j] 0]
			set lignes($PREVIOUSI) [lreplace $lignes($PREVIOUSI) [expr $PREVIOUSJ] [expr $PREVIOUSJ] 0]
	
			# mise a jour du compteur de materiel
			incr WHITES -1
	
			## update global variables
			set LASTMOVE "[charCol $PREVIOUSJ]$PREVIOUSI[charCol $j]$i$PRB$PRN$GRB$GRN" 
			set STRMOVE "[charCol $PREVIOUSJ][expr $PREVIOUSI+1][charCol $j][expr $i+1]"
			set PREVIOUSC $coup
			set PREVIOUSI -1
			set PREVIOUSJ -1
			set COLOR 1	
			return true
		} else {
			## wrong move
			if {($i+$j)%2} { 
				.plateau.r$i.b$i$j configure -background [EmptyBG]
			} else {
				.plateau.r$i.b$i$j configure -background [ButtonBG]
			}
			set PREVIOUSI -1
			set PREVIOUSJ -1
			return false
		}
	}
	## promotions
	if {[expr ($i == 0)] && [expr ([nState $PREVIOUSI $PREVIOUSJ] ==-1)]} {
		global COMPUTERMOVE
		if {$COMPUTERMOVE <= 0} {
			set coordI $i
			set coordJ $j
			Promotion 
			return true
		} else {
			set piecePromue [expr -[string index $CURRMOVE 8]]
			# mise a jour du compteur de materiel
			if {[expr ($piecePromue == -5)]} {incr BLACKS 8}
			if {[expr ($piecePromue == -4)]} {incr BLACKS 4}
			if {[expr ($piecePromue == -3)]} {incr BLACKS 2}
			if {[expr ($piecePromue == -2)]} {incr BLACKS 2}
	
			## modify the data structure
			set lignes($i) [lreplace $lignes($i) [expr $j] [expr $j] $piecePromue]
			set lignes($PREVIOUSI) [lreplace $lignes($PREVIOUSI) [expr $PREVIOUSJ] [expr $PREVIOUSJ] 0]
	
			## display the new configuration
			if {($i+$j)%2} { 
				.plateau.r$i.b$i$j configure -image [sState $piecePromue] -background [EmptyBG]
			} else {
				.plateau.r$i.b$i$j configure -image [sState $piecePromue] -background [ButtonBG] 
			}
			if {($PREVIOUSI+$PREVIOUSJ)%2} { 
				.plateau.r$PREVIOUSI.b$PREVIOUSI$PREVIOUSJ configure -image [sState 0] -background [EmptyBG]
			} else {
				.plateau.r$PREVIOUSI.b$PREVIOUSI$PREVIOUSJ configure -image [sState 0] -background [ButtonBG] 
			}
	
			## update globals
			set LASTMOVE "[charCol $PREVIOUSJ]$PREVIOUSI[charCol $j]$i$PRB$PRN$GRB$GRN" 
			set STRMOVE "[charCol $PREVIOUSJ][expr $PREVIOUSI+1][charCol $j][expr $i+1]"
			set PREVIOUSC $coup
			set PREVIOUSI -1
			set PREVIOUSJ -1
			set COLOR 1	
			return true
		}	
	}
	if {[expr ($i == 7)] && [expr ([nState $PREVIOUSI $PREVIOUSJ] ==1)]} {
		global COMPUTERMOVE
		if {$COMPUTERMOVE <= 0} {
			set coordI $i
			set coordJ $j
			Promotion
			return true
		} else {
			set piecePromue [expr [string index $CURRMOVE 8]]
			# mise a jour du compteur de materiel
			if {[expr ($piecePromue == 5)]} {incr WHITES 8}
			if {[expr ($piecePromue == 4)]} {incr WHITES 4}
			if {[expr ($piecePromue == 3)]} {incr WHITES 2}
			if {[expr ($piecePromue == 2)]} {incr WHITES 2}
	
			## modify the data structure
			set lignes($i) [lreplace $lignes($i) [expr $j] [expr $j] $piecePromue]
			set lignes($PREVIOUSI) [lreplace $lignes($PREVIOUSI) [expr $PREVIOUSJ] [expr $PREVIOUSJ] 0]
				
			## display the new configuration
			if {($i+$j)%2} { 
				.plateau.r$i.b$i$j configure -image [sState $piecePromue] -background [EmptyBG]
			} else {
				.plateau.r$i.b$i$j configure -image [sState $piecePromue] -background [ButtonBG] 
			}
			if {($PREVIOUSI+$PREVIOUSJ)%2} { 
				.plateau.r$PREVIOUSI.b$PREVIOUSI$PREVIOUSJ configure -image [sState 0] -background [EmptyBG]
			} else {
				.plateau.r$PREVIOUSI.b$PREVIOUSI$PREVIOUSJ configure -image [sState 0] -background [ButtonBG] 
			}
	
			## update globals
			set LASTMOVE "[charCol $PREVIOUSJ]$PREVIOUSI[charCol $j]$i$PRB$PRN$GRB$GRN" 
			set STRMOVE "[charCol $PREVIOUSJ][expr $PREVIOUSI+1][charCol $j][expr $i+1]"
			set PREVIOUSC $coup
			set PREVIOUSI -1
			set PREVIOUSJ -1
			set COLOR -1	
			return true
		}
	} 
	
	################
	## Coup regulier
	################
	
	# recherche de l'emplacement du roi
	for {set k 0} { [expr $k < 8] } {incr k} {
		for {set l 0} { [expr $l < 8] } {incr l} {
			set currState [nState $k $l]
			if {[expr $COLOR * $currState] == 6} {
				set placeRoix $k
				set placeRoiy $l
				break
			}
		}
	}
	
	# verification 
	set correct 0
	if {[ABS $caseDepart] == 1} {
		## pion
		if {$COLOR<0 && [expr [expr $i+1] == $PREVIOUSI] && ([expr [expr $j-$PREVIOUSJ] == 0] || ([expr [ABS [expr $j-$PREVIOUSJ]] == 1] && [expr $coup != 0]))} {
			## modify the data structure
			set lignes($i) [lreplace $lignes($i) [expr $j] [expr $j] $caseDepart]
			set lignes($PREVIOUSI) [lreplace $lignes($PREVIOUSI) [expr $PREVIOUSJ] [expr $PREVIOUSJ] 0]
			if {! [UnderCheck $placeRoix $placeRoiy]} {
				set correct 1
			} else {
				## takeback
				set lignes($i) [lreplace $lignes($i) [expr $j] [expr $j] $coup]
				set lignes($PREVIOUSI) [lreplace $lignes($PREVIOUSI) [expr $PREVIOUSJ] [expr $PREVIOUSJ] $caseDepart]
			}		
		}
		if {$COLOR<0 && [expr [expr $i+2] == $PREVIOUSI] && ([expr [expr $j-$PREVIOUSJ] == 0])} {
			## modify the data structure
			set lignes($i) [lreplace $lignes($i) [expr $j] [expr $j] $caseDepart]
			set lignes($PREVIOUSI) [lreplace $lignes($PREVIOUSI) [expr $PREVIOUSJ] [expr $PREVIOUSJ] 0]
			if {! [UnderCheck $placeRoix $placeRoiy]} {
				set correct 1
			} else {
				## takeback
				set lignes($i) [lreplace $lignes($i) [expr $j] [expr $j] $coup]
				set lignes($PREVIOUSI) [lreplace $lignes($PREVIOUSI) [expr $PREVIOUSJ] [expr $PREVIOUSJ] $caseDepart]
			}
		}
		if {$COLOR>0 && [expr [expr $i-1] == $PREVIOUSI] && ([expr [expr $j-$PREVIOUSJ] == 0] || ([expr [ABS [expr $j-$PREVIOUSJ]] == 1] && [expr $coup != 0]))} {
			## modify the data structure
			set lignes($i) [lreplace $lignes($i) [expr $j] [expr $j] $caseDepart]
			set lignes($PREVIOUSI) [lreplace $lignes($PREVIOUSI) [expr $PREVIOUSJ] [expr $PREVIOUSJ] 0]
			if {! [UnderCheck $placeRoix $placeRoiy]} {
				set correct 1
			} else {
				## takeback
				set lignes($i) [lreplace $lignes($i) [expr $j] [expr $j] $coup]
				set lignes($PREVIOUSI) [lreplace $lignes($PREVIOUSI) [expr $PREVIOUSJ] [expr $PREVIOUSJ] $caseDepart]
			}
		}
		if {$COLOR>0 && [expr [expr $i-2] == $PREVIOUSI] && ([expr [expr $j-$PREVIOUSJ] == 0])} {
			## modify the data structure
			set lignes($i) [lreplace $lignes($i) [expr $j] [expr $j] $caseDepart]
			set lignes($PREVIOUSI) [lreplace $lignes($PREVIOUSI) [expr $PREVIOUSJ] [expr $PREVIOUSJ] 0]
			if {! [UnderCheck $placeRoix $placeRoiy]} {
				set correct 1
			} else {
				## takeback
				set lignes($i) [lreplace $lignes($i) [expr $j] [expr $j] $coup]
				set lignes($PREVIOUSI) [lreplace $lignes($PREVIOUSI) [expr $PREVIOUSJ] [expr $PREVIOUSJ] $caseDepart]
			}
		}
	}
	if {[ABS $caseDepart] == 2} {
		## cavalier
		if {[expr [expr $i-2] == $PREVIOUSI] && [expr [ABS [expr $j-$PREVIOUSJ]] == 1]} {
			## modify the data structure
			set lignes($i) [lreplace $lignes($i) [expr $j] [expr $j] $caseDepart]
			set lignes($PREVIOUSI) [lreplace $lignes($PREVIOUSI) [expr $PREVIOUSJ] [expr $PREVIOUSJ] 0]
			if {! [UnderCheck $placeRoix $placeRoiy]} {
				set correct 1
			} else {
				## takeback
				set lignes($i) [lreplace $lignes($i) [expr $j] [expr $j] $coup]
				set lignes($PREVIOUSI) [lreplace $lignes($PREVIOUSI) [expr $PREVIOUSJ] [expr $PREVIOUSJ] $caseDepart]
			}
		}
		if {[expr [expr $i+2] == $PREVIOUSI] && [expr [ABS [expr $j-$PREVIOUSJ]] == 1]} {
			## modify the data structure
			set lignes($i) [lreplace $lignes($i) [expr $j] [expr $j] $caseDepart]
			set lignes($PREVIOUSI) [lreplace $lignes($PREVIOUSI) [expr $PREVIOUSJ] [expr $PREVIOUSJ] 0]
			if {! [UnderCheck $placeRoix $placeRoiy]} {
				set correct 1
			} else {
				## takeback
				set lignes($i) [lreplace $lignes($i) [expr $j] [expr $j] $coup]
				set lignes($PREVIOUSI) [lreplace $lignes($PREVIOUSI) [expr $PREVIOUSJ] [expr $PREVIOUSJ] $caseDepart]
			}
		}
		if {[expr [expr $j-2] == $PREVIOUSJ] && [expr [ABS [expr $i-$PREVIOUSI]] == 1]} {
			## modify the data structure
			set lignes($i) [lreplace $lignes($i) [expr $j] [expr $j] $caseDepart]
			set lignes($PREVIOUSI) [lreplace $lignes($PREVIOUSI) [expr $PREVIOUSJ] [expr $PREVIOUSJ] 0]
			if {! [UnderCheck $placeRoix $placeRoiy]} {
				set correct 1
			} else {
				## takeback
				set lignes($i) [lreplace $lignes($i) [expr $j] [expr $j] $coup]
				set lignes($PREVIOUSI) [lreplace $lignes($PREVIOUSI) [expr $PREVIOUSJ] [expr $PREVIOUSJ] $caseDepart]
			}
		}
		if {[expr [expr $j+2] == $PREVIOUSJ] && [expr [ABS [expr $i-$PREVIOUSI]] == 1]} {
			## modify the data structure
			set lignes($i) [lreplace $lignes($i) [expr $j] [expr $j] $caseDepart]
			set lignes($PREVIOUSI) [lreplace $lignes($PREVIOUSI) [expr $PREVIOUSJ] [expr $PREVIOUSJ] 0]
			if {! [UnderCheck $placeRoix $placeRoiy]} {
				set correct 1
			} else {
				## takeback
				set lignes($i) [lreplace $lignes($i) [expr $j] [expr $j] $coup]
				set lignes($PREVIOUSI) [lreplace $lignes($PREVIOUSI) [expr $PREVIOUSJ] [expr $PREVIOUSJ] $caseDepart]
			}
		}
	}
	if {[ABS $caseDepart] == 3} {
		## fou
		if {[expr $i - $PREVIOUSI] == [expr $j - $PREVIOUSJ]} { 
			if {$PREVIOUSI < $i} {
				set caseInitx [expr $PREVIOUSI + 1]
				set caseFinx [expr $i - 1]
				set caseInity [expr $PREVIOUSJ + 1]
			} else {
				set caseInitx [expr $i + 1]
				set caseFinx [expr $PREVIOUSI - 1]
				set caseInity [expr $j + 1]
			}	
			set x $caseInitx
			set y $caseInity
			for {} { [expr $x <= $caseFinx ] } {incr x ; incr y} {
				if {[nState $x $y] != 0} {break}
			}
			if {$x > $caseFinx} {
				## modify the data structure
				set lignes($i) [lreplace $lignes($i) [expr $j] [expr $j] $caseDepart]
				set lignes($PREVIOUSI) [lreplace $lignes($PREVIOUSI) [expr $PREVIOUSJ] [expr $PREVIOUSJ] 0]
				if {! [UnderCheck $placeRoix $placeRoiy]} {
					set correct 1
				} else {
					## takeback
					set lignes($i) [lreplace $lignes($i) [expr $j] [expr $j] $coup]
					set lignes($PREVIOUSI) [lreplace $lignes($PREVIOUSI) [expr $PREVIOUSJ] [expr $PREVIOUSJ] $caseDepart]
				}
			}
		}
		if {[expr $i - $PREVIOUSI] == -[expr $j - $PREVIOUSJ]} { 
			if {$PREVIOUSI < $i} {
				set caseInitx [expr $i - 1]
				set caseFinx [expr $PREVIOUSI + 1]
				set caseInity [expr $j + 1]
			} else {
				set caseInitx [expr $PREVIOUSI - 1]
				set caseFinx [expr $i + 1]
				set caseInity [expr $PREVIOUSJ + 1]
			}	
			set x $caseInitx
			set y $caseInity
			for {} { [expr $x >= $caseFinx ] } {incr x -1 ; incr y} {
				if {[nState $x $y] != 0} {break}
			}
			if {$x < $caseFinx} {
				## modify the data structure
				set lignes($i) [lreplace $lignes($i) [expr $j] [expr $j] $caseDepart]
				set lignes($PREVIOUSI) [lreplace $lignes($PREVIOUSI) [expr $PREVIOUSJ] [expr $PREVIOUSJ] 0]
				if {! [UnderCheck $placeRoix $placeRoiy]} {
					set correct 1
				} else {
					## takeback
					set lignes($i) [lreplace $lignes($i) [expr $j] [expr $j] $coup]
					set lignes($PREVIOUSI) [lreplace $lignes($PREVIOUSI) [expr $PREVIOUSJ] [expr $PREVIOUSJ] $caseDepart]
				}
			}
		}
	}
	if {[ABS $caseDepart] == 4} {
		## tour
		if {$PREVIOUSI == $i} {
			if {$PREVIOUSJ < $j} {
				set caseInit [expr $PREVIOUSJ + 1]
				set caseFin [expr $j - 1]
			} else {
				set caseInit [expr $j + 1]
				set caseFin [expr $PREVIOUSJ - 1]
			}	
			set x $caseInit
			for {} { [expr $x <= $caseFin ] } {incr x} {
				if {[nState $PREVIOUSI $x] != 0} {break}
			}
			if {$x > $caseFin} {
				## modify the data structure
				set lignes($i) [lreplace $lignes($i) [expr $j] [expr $j] $caseDepart]
				set lignes($PREVIOUSI) [lreplace $lignes($PREVIOUSI) [expr $PREVIOUSJ] [expr $PREVIOUSJ] 0]
				if {! [UnderCheck $placeRoix $placeRoiy]} {
					set correct 1
				} else {
					## takeback
					set lignes($i) [lreplace $lignes($i) [expr $j] [expr $j] $coup]
					set lignes($PREVIOUSI) [lreplace $lignes($PREVIOUSI) [expr $PREVIOUSJ] [expr $PREVIOUSJ] $caseDepart]
				}
			}
		}
		if {$PREVIOUSJ == $j} {
			if {$PREVIOUSI < $i} {
				set caseInit [expr $PREVIOUSI + 1]
				set caseFin [expr $i - 1]
			} else {
				set caseInit [expr $i + 1]
				set caseFin [expr $PREVIOUSI - 1]
			}	
			set x $caseInit
			for {} { [expr $x <= $caseFin ] } {incr x} {
				if {[nState $x $PREVIOUSJ] != 0} {break}
			}
			if {$x > $caseFin} {
				## modify the data structure
				set lignes($i) [lreplace $lignes($i) [expr $j] [expr $j] $caseDepart]
				set lignes($PREVIOUSI) [lreplace $lignes($PREVIOUSI) [expr $PREVIOUSJ] [expr $PREVIOUSJ] 0]
				if {! [UnderCheck $placeRoix $placeRoiy]} {
					set correct 1
				} else {
					## takeback
					set lignes($i) [lreplace $lignes($i) [expr $j] [expr $j] $coup]
					set lignes($PREVIOUSI) [lreplace $lignes($PREVIOUSI) [expr $PREVIOUSJ] [expr $PREVIOUSJ] $caseDepart]
				}
			}
		}
	}
	if {[ABS $caseDepart] == 5} {
		## dame
		if {[expr $i - $PREVIOUSI] == [expr $j - $PREVIOUSJ]} { 
			if {$PREVIOUSI < $i} {
				set caseInitx [expr $PREVIOUSI + 1]
				set caseFinx [expr $i - 1]
				set caseInity [expr $PREVIOUSJ + 1]
			} else {
				set caseInitx [expr $i + 1]
				set caseFinx [expr $PREVIOUSI - 1]
				set caseInity [expr $j + 1]
			}	
			set x $caseInitx
			set y $caseInity
			for {} { [expr $x <= $caseFinx ] } {incr x ; incr y} {
				if {[nState $x $y] != 0} {break}
			}
			if {$x > $caseFinx} {
				## modify the data structure
				set lignes($i) [lreplace $lignes($i) [expr $j] [expr $j] $caseDepart]
				set lignes($PREVIOUSI) [lreplace $lignes($PREVIOUSI) [expr $PREVIOUSJ] [expr $PREVIOUSJ] 0]
				if {! [UnderCheck $placeRoix $placeRoiy]} {
					set correct 1
				} else {
					## takeback
					set lignes($i) [lreplace $lignes($i) [expr $j] [expr $j] $coup]
					set lignes($PREVIOUSI) [lreplace $lignes($PREVIOUSI) [expr $PREVIOUSJ] [expr $PREVIOUSJ] $caseDepart]
				}
			}
		}
		if {[expr $i - $PREVIOUSI] == -[expr $j - $PREVIOUSJ]} { 
			if {$PREVIOUSI < $i} {
				set caseInitx [expr $i - 1]
				set caseFinx [expr $PREVIOUSI + 1]
				set caseInity [expr $j + 1]
			} else {
				set caseInitx [expr $PREVIOUSI - 1]
				set caseFinx [expr $i + 1]
				set caseInity [expr $PREVIOUSJ + 1]
			}	
			set x $caseInitx
			set y $caseInity
			for {} { [expr $x >= $caseFinx ] } {incr x -1 ; incr y} {
				if {[nState $x $y] != 0} {break}
			}
			if {$x < $caseFinx} {
				## modify the data structure
				set lignes($i) [lreplace $lignes($i) [expr $j] [expr $j] $caseDepart]
				set lignes($PREVIOUSI) [lreplace $lignes($PREVIOUSI) [expr $PREVIOUSJ] [expr $PREVIOUSJ] 0]
				if {! [UnderCheck $placeRoix $placeRoiy]} {
					set correct 1
				} else {
					## takeback
					set lignes($i) [lreplace $lignes($i) [expr $j] [expr $j] $coup]
					set lignes($PREVIOUSI) [lreplace $lignes($PREVIOUSI) [expr $PREVIOUSJ] [expr $PREVIOUSJ] $caseDepart]
				}
			}
		}
		if {$PREVIOUSI == $i} {
			if {$PREVIOUSJ < $j} {
				set caseInit [expr $PREVIOUSJ + 1]
				set caseFin [expr $j - 1]
			} else {
				set caseInit [expr $j + 1]
				set caseFin [expr $PREVIOUSJ - 1]
			}	
			set x $caseInit
			for {} { [expr $x <= $caseFin ] } {incr x} {
				if {[nState $PREVIOUSI $x] != 0} {break}
			}
			if {$x > $caseFin} {
				## modify the data structure
				set lignes($i) [lreplace $lignes($i) [expr $j] [expr $j] $caseDepart]
				set lignes($PREVIOUSI) [lreplace $lignes($PREVIOUSI) [expr $PREVIOUSJ] [expr $PREVIOUSJ] 0]
				if {! [UnderCheck $placeRoix $placeRoiy]} {
					set correct 1
				} else {
					## takeback
					set lignes($i) [lreplace $lignes($i) [expr $j] [expr $j] $coup]
					set lignes($PREVIOUSI) [lreplace $lignes($PREVIOUSI) [expr $PREVIOUSJ] [expr $PREVIOUSJ] $caseDepart]
				}
			}
		}
		if {$PREVIOUSJ == $j} {
			if {$PREVIOUSI < $i} {
				set caseInit [expr $PREVIOUSI + 1]
				set caseFin [expr $i - 1]
			} else {
				set caseInit [expr $i + 1]
				set caseFin [expr $PREVIOUSI - 1]
			}	
			set x $caseInit
			for {} { [expr $x <= $caseFin ] } {incr x} {
				if {[nState $x $PREVIOUSJ] != 0} {break}
			}
			if {$x > $caseFin} {
				## modify the data structure
				set lignes($i) [lreplace $lignes($i) [expr $j] [expr $j] $caseDepart]
				set lignes($PREVIOUSI) [lreplace $lignes($PREVIOUSI) [expr $PREVIOUSJ] [expr $PREVIOUSJ] 0]
				if {! [UnderCheck $placeRoix $placeRoiy]} {
					set correct 1
				} else {
					## takeback
					set lignes($i) [lreplace $lignes($i) [expr $j] [expr $j] $coup]
					set lignes($PREVIOUSI) [lreplace $lignes($PREVIOUSI) [expr $PREVIOUSJ] [expr $PREVIOUSJ] $caseDepart]
				}
			}
		}
	}
	if {[ABS $caseDepart] == 6} {
		## roi
		if {[expr [ABS [expr $i-$PREVIOUSI]] <= 1] && [expr [ABS [expr $j-$PREVIOUSJ]] <= 1]} {
			## modify the data structure
			set lignes($i) [lreplace $lignes($i) [expr $j] [expr $j] $caseDepart]
			set lignes($PREVIOUSI) [lreplace $lignes($PREVIOUSI) [expr $PREVIOUSJ] [expr $PREVIOUSJ] 0]
			if {! [UnderCheck $i $j]} {
				set correct 1
			} else {
				## takeback
				set lignes($i) [lreplace $lignes($i) [expr $j] [expr $j] $coup]
				set lignes($PREVIOUSI) [lreplace $lignes($PREVIOUSI) [expr $PREVIOUSJ] [expr $PREVIOUSJ] $caseDepart]
			}
		}
	}
	
	if { $correct == 1} {
		### Valid move
		# mise a jour du compteur de materiel
		if {[expr ($coup == 5)]} {incr WHITES -9}
		if {[expr ($coup == 4)]} {incr WHITES -5}
		if {[expr ($coup == 3)]} {incr WHITES -3}
		if {[expr ($coup == 2)]} {incr WHITES -3}
		if {[expr ($coup == 1)]} {incr WHITES -1}
		if {[expr ($coup == -5)]} {incr BLACKS -9}
		if {[expr ($coup == -4)]} {incr BLACKS -5}
		if {[expr ($coup == -3)]} {incr BLACKS -3}
		if {[expr ($coup == -2)]} {incr BLACKS -3}
		if {[expr ($coup == -1)]} {incr BLACKS -1}
	
		## update global flags
		if {[expr $caseDepart == 6]} {
			set PRB 0
			set GRB 0
		}
		if {[expr $caseDepart == -6]} {
			set PRN 0
			set GRN 0
		}
		if {[expr $PREVIOUSI == 0] && [expr $PREVIOUSJ == 0]} {set GRB 0}
		if {[expr $PREVIOUSI == 0] && [expr $PREVIOUSJ == 7]} {set PRB 0}
		if {[expr $PREVIOUSI == 7] && [expr $PREVIOUSJ == 0]} {set GRN 0}
		if {[expr $PREVIOUSI == 7] && [expr $PREVIOUSJ == 7]} {set PRN 0}
	
		## display the new configuration
		if {($i+$j)%2} { 
			.plateau.r$i.b$i$j configure -image [sState $caseDepart] -background [EmptyBG]
		} else {
			.plateau.r$i.b$i$j configure -image [sState $caseDepart] -background [ButtonBG] 
		}
		if {($PREVIOUSI+$PREVIOUSJ)%2} { 
			.plateau.r$PREVIOUSI.b$PREVIOUSI$PREVIOUSJ configure -image [sState 0] -background [EmptyBG]
		} else {
			.plateau.r$PREVIOUSI.b$PREVIOUSI$PREVIOUSJ configure -image [sState 0] -background [ButtonBG] 
		}
		
		## update global variables
		set LASTMOVE "[charCol $PREVIOUSJ]$PREVIOUSI[charCol $j]$i$PRB$PRN$GRB$GRN" 
		set STRMOVE "[charCol $PREVIOUSJ][expr $PREVIOUSI+1][charCol $j][expr $i+1]"
		set PREVIOUSC $coup
		set PREVIOUSI -1
		set PREVIOUSJ -1
		set COLOR [expr -$COLOR]
		
		if {($coup != 0) || ([ABS $caseDepart] == 1)} {set CINQUANTE 0
		} else {set CINQUANTE [expr $CINQUANTE + 1]}
		if {$CINQUANTE >= 100} {
			Quit "Regle des 50 coups : match nul"
		}
		 
		return true
	} else {
		## wrong move
		if {($PREVIOUSI+$PREVIOUSJ)%2} {
			.plateau.r$PREVIOUSI.b$PREVIOUSI$PREVIOUSJ configure -background [EmptyBG]
		} else {
			.plateau.r$PREVIOUSI.b$PREVIOUSI$PREVIOUSJ configure -background [ButtonBG]
		}
		set PREVIOUSI -1
		set PREVIOUSJ -1
		return false
	}
}

# attend un coup adverse, et memorise le temps passe a la fin de celui-ci
proc WaitPlay {} { 
	global PROG PLATEAU FILE JOUEUR ARGS
	global LASTMOVE
	global CURRMOVE 
	global COLOR
	global CINQUANTE
	global OVER

	## Sends board to file
	DisplayGame $JOUEUR

	set GO [clock clicks]
	set CURRMOVE [Execute $PROG($JOUEUR) $PLATEAU $ARGS($JOUEUR) $LASTMOVE]
	
	if {([string index $CURRMOVE 0] == [string index $CURRMOVE 2]) && ([string index $CURRMOVE 1] == 0) && ([string index $CURRMOVE 3] == 0)} {
		set OVER $COLOR
		Quit "Echec et mat"
	}
	if {([string index $CURRMOVE 0] == [string index $CURRMOVE 2]) && ([string index $CURRMOVE 1] == 7) && ([string index $CURRMOVE 3] == 7)} {
		set OVER 0
		Quit "Pat : partie nulle"
	}

	set caseDepart [nState [string index $CURRMOVE 1] [intCol [string index $CURRMOVE 0]]]
	set coup [nState [string index $CURRMOVE 3] [intCol [string index $CURRMOVE 2]]]
	if {($coup != 0) || ([ABS $caseDepart] == 1)} {set CINQUANTE 0
	} else {set CINQUANTE [expr $CINQUANTE + 1]}
	if {$CINQUANTE >= 100} {
		set OVER 0
		Quit "Regle des 50 coups : match nul"
	}

	set END [clock clicks]
	set FILEID [open $FILE($JOUEUR) a 0666]
	puts $FILEID $CURRMOVE 
	close $FILEID
	return [expr $END - $GO]
}

## Next player
proc Next {} { 
	set MAXNBCOUP 999
	global NBCOUPS
	if { [expr  ($NBCOUPS(1) > $MAXNBCOUP) && ($NBCOUPS(-1) > $MAXNBCOUP)]} {
		Quit "Max. Coups."; return
	}

	global OVER
	if {! $OVER} {
		global CURRMOVE
		global COMPUTING
		global ORDI 
		global JOUEUR
		global WHITES 
		global BLACKS
		global prevCoordI
		global COMPUTERMOVE
		set JOUEUR [expr -($JOUEUR)]
		## Computer game
		if {$ORDI} {
			if {$ORDI != $JOUEUR} {
				set COMPUTERMOVE 1
				set COMPUTING "analyse ..."
				## Write preceeding move
				update idletasks
		
				if ($prevCoordI>=0) {
					set z 0; after 60000 { set z 1 } ; vwait prevCoordI
				}
				set TIMING [WaitPlay]
				set COMPUTING "joue ..."   
				TreatTime $TIMING

				set x 0
				set y 0
				set final false 
				for {set i 0} { [expr $i < 4] } {incr i} {
					## sleep 1
					set z 0; after 500 { set z 1 } ; vwait z
					set y [intCol [string index $CURRMOVE $i]]
					if {[expr ($y < 0)]} { Quit "Arrt."; return }
					incr i
					set x [string index $CURRMOVE $i]
					if {! [string match {[0-9]} $x]} { Quit "Arrt."; return }
					set final [Select $x $y ]
					update idletasks
				} 
				set COMPUTING "" 
				set COMPUTERMOVE 0
				update idletasks
				TreatLate $TIMING
				Next 
			} else {
				## Write preceeding move
				## but not to screen
				update idletasks
			}
		} else {
			## Write preceeding move
			update idletasks
		}
	
	} else {
		## Is over
		Quit "Fin."
	}
}

###########################
# Fin du jeu : destruction
###########################

# Removes squares
proc Destroy {} {
	global DIM
	for {set i 0} {[expr $i < $DIM]} {incr i} {
		for {set j 0} {[expr $j < $DIM]} {incr j} {
			if {($i+$j)%2} { 
				destroy .plateau.r$i.c$i$j
			} else { 
				destroy .plateau.r$i.b$i$j
			}	       
		}
		destroy .plateau.r$i
	}	    
}

# procedure permettant d'arreter le jeu si un des camps a gagne ou si c'est nulle
proc Quit { msg } { 
	global COMPUTING
	set COMPUTING $msg
	update idletasks
	global OVER 
	
	if {! $OVER} {
		#late
		global RETARD
		set difflate [expr $RETARD(1) - $RETARD(-1)]
		if {[expr $difflate > 0]} {
			set OVER -1
		} else {
			if {[expr $difflate < 0]} {
				set OVER 1
			}
		}
	} 
	if {[expr $OVER == 1]} {
		puts "Partie terminee. Vainqueur : NOIRS."
	} else {
		if {[expr $OVER == -1]} {
			puts "Partie terminee. Vainqueur : BLANCS."
		} else {
			puts "Partie terminee. Egalite."
		}
	}	
}

###################
# Gestion du temps
###################

# met a jour les temps
proc TreatTime { ltime } {
	global GLOBALTIME
	global GLOBALLAST
	global NBCOUPS
	global JOUEUR

	incr NBCOUPS($JOUEUR)
	set GLOBALLAST($JOUEUR) [expr $ltime / 1000000.0]
	set GLOBALTIME($JOUEUR) [expr $GLOBALTIME($JOUEUR) + [expr $ltime / 1000000.0] ]
}

# traite les retards
proc TreatLate { ltime } {
	global ORDI
	if {[expr $ORDI == -2]} {
		global RETARD
		global MAXTIME
		global JOUEUR

		global BIGLATE
		set BIGFACTOR 3
		set BIGMAC    2
		set MAXRET    300
		set MAXDIFF   200
	
		if {[expr ($ltime> $MAXTIME($JOUEUR))]} {
			incr RETARD($JOUEUR)
			if {[expr ($ltime> [expr $BIGFACTOR * $MAXTIME($JOUEUR)])]} {
				incr BIGLATE($JOUEUR)
				if {[expr $BIGLATE($JOUEUR) > $BIGMAC]} {
					global OVER; set OVER $JOUEUR
					if {[expr $JOUEUR == 1]} {
						Quit "Blancs DISQ."; return 
					} else {
						Quit "Noirs DISQ." ; return 
					}
				}
			}
			if {[expr  ($RETARD($JOUEUR) > $MAXRET) || ($RETARD($JOUEUR) - $RETARD([expr - $JOUEUR]) > $MAXDIFF)  ]} {
				global OVER; set OVER $JOUEUR
				if {[expr $JOUEUR == 1]} {
					Quit "Blancs DISQ." ; return 
				} else {
					Quit "Noirs DISQ." ; return 
				}
			}
		}
	}
}

Init

