#ifndef _AlphaBeta
#define _AlphaBeta

/*
* Classe alpha-beta
*/

#include "Coup.h"
#include "GlobalConstants.h"
#include "Plateau.h"
#include "IterateurCoupW.h"
#include "IterateurCoupB.h"
#include "UnderCheck.h"
#include "Echanges.h"
#include "Timer.h"
#include <vector>
#include <cstdlib>
#include <algorithm>
#include <unistd.h> /* Pour getpid */
#include <iostream>

#include "VectorSignEval.h"

/* lorsque l'on supprimera les vecteurs on n'aura plus besoin de distinguer B et W */

/* structure avec une valeur et un indice, servant pour les tris */
struct ValIndiceW {
	ValIndiceW () {}
	ValIndiceW (int ind, float ev) {
		indice=ind;
		eval=ev;
	}
	bool operator< (const ValIndiceW& valI) const {
		return (eval<valI.eval);
	}
	int indice;
	float eval;
};
struct ValIndiceB {
	ValIndiceB () {}
	ValIndiceB (int ind, float ev) {
		indice=ind;
		eval=ev;
	}
	bool operator< (const ValIndiceB& valI) const {
		return (eval>valI.eval);
	}
	int indice;
	float eval;
};

/* structure consistant en un coup + son evaluation */
struct EvalCoupW {
	EvalCoupW () {}
	EvalCoupW (const EvalCoupW& evcp) {
		cp=evcp.cp;
		eval=evcp.eval;
	}
	EvalCoupW (Coup c,float ev) {
		cp=c;
		eval=ev;
	}
	bool operator<(const EvalCoupW& evcp) const {
		return (eval<evcp.eval); 
	}
	Coup cp;
	float eval;
};
struct EvalCoupB {
	EvalCoupB () {}
	EvalCoupB (const EvalCoupB& evcp) {
		cp=evcp.cp;
		eval=evcp.eval;
	}
	EvalCoupB (Coup c,float ev) {
		cp=c;
		eval=ev;
	}
	bool operator<(const EvalCoupB& evcp) const {
		return (eval>evcp.eval); 
	}
	Coup cp;
	float eval;
};

/* structure avec une valeur et un indice, servant pour les tris a faible profondeur */
struct HeurValInd {
	HeurValInd () {}
	HeurValInd (int ind, int val) {
		indice=ind;
		valeur=val;
	}
	bool operator< (const HeurValInd& hVal) const {
		return (valeur<hVal.valeur);
	}
	int indice;
	int valeur;
};      

class AlphaBeta {
      
public:
       
	/* constructeurs */
	AlphaBeta() {}
       
	/* construction a partir d'un temps */
	AlphaBeta(double tp) {
		tmp=tp;
		/* calcul de la taille des tables de hash en fonction du temps alloue */
		if (tp>7) TAILLE_MAX=700000;
		else TAILLE_MAX=(int)((tp/7)*700000);
	}

	/* une petite fonction qui permet d'analyser moins loin les "mauvais" coups */
	int floorpp(float f) { 
		/* calcule la partie entiere de la partie positive d'un nombre reel */
		if (f<=0) return 0;
		else return (int)f;
	}    
	
	/* fonction renvoyant le coup dsign par alphaBeta, pour les blancs */
	Coup coupAB_W();

	/* fonction renvoyant le coup dsign par alphaBeta, pour les noirs */
	Coup coupAB_B();

	/* les fonctions alpha-beta: renvoient l'evaluation a la profondeur demandee */
	float alphaBetaProf_1W(float A,float B);
	float alphaBetaProf_1B(float A,float B);
	float alphaBetaProf_2W(float A,float B);
	float alphaBetaProf_2B(float A,float B);
	float alphaBetaW(int pf,float A,float B);
	float alphaBetaB(int pf,float A,float B);

	/* permutations aleatoires de bons coups */
	void random_permutW(std::vector<EvalCoupW>& evcp,int nbElts) const;
	void random_permutB(std::vector<EvalCoupB>& evcp,int nbElts) const;

	/* trie des ensembles de coups en preservant l'ordre */
	void tri_ordonneW(std::vector<EvalCoupW>& ord) const;
	void tri_ordonneB(std::vector<EvalCoupB>& ord) const;

	/* tri casiers lineaire */
	void triLin(HeurValInd* valInd,int sz);

	/* destructeur */
	~AlphaBeta() {}
      
private:
        
	/* temps */
	double tmp;	
	Timer tim;
			
	bool stopped; /* indique si on doit arreter le calcul */
	int prof; /* profondeur courante */
	/* booleens indiquant si on est en train d'analyser le 1er ou 2eme coup d'un etage */
	bool firstMove,secondMove;

	/* taille maxi des tables (fonction du temps dont on dispose) */
	unsigned int TAILLE_MAX;
	
	/* taille courante des tables */
	unsigned int tailleTablesW,tailleTablesB;
       
	/* on memorise les plateaux+trait (+ si b, - si n) dans les alpha-beta, par niveau.
	* avant chaque recherche d'alpha-beta, on cherche donc dans le bon tableau si la solution s'y trouve */
	VectorSignEval bestProfW[NB_MAX],bestProfB[NB_MAX];
	
	/* memorisation des codes positions rencontrees au cours d'une descente dans l'arbre */
	int* repetPosW;
	int* repetPosB;
	
};

#endif
