#ifndef _Plateau
#define _Plateau

#include "Coup.h"
#include "GlobalConstants.h"
#include <cmath>
#include <cstdlib>
#include <string>
#include <fstream>

/* structure contenant la signature du plateau + cle */
struct Signature {
	Signature () {}       
	Signature (std::string& s, int cle) {
		chaine=s;
		clef=cle;
	}
	std::string chaine;
	int clef;
};

/* structure contenant la signature du plateau + profondeur + evaluation (cf hachage) */
struct SignEval {
	SignEval () {}
	SignEval (int pf, float ev, unsigned long long cle) {
		prof=pf;
		eval=ev;
		clef=cle;
	}
	int prof;
	float eval;
	unsigned long long clef;
};

/* structure contenant la clef de base + raffinee du plateau (cf hachage) */
struct Clefs {
	Clefs (unsigned int kb, unsigned long long kr) {
		keyB=kb;
		keyR=kr;
	}
	unsigned int keyB;
	unsigned long long keyR;
};

/*Stockage du plateau */

class Plateau {

public:
       
	Plateau() {}

	/* Construction du plateau a partir d'un fichier */
	Plateau(char* name);
    	  
	/* accs  une case du tableau reprsentant le plateau */
	int operator[](int i) const { 
		return board[i];
	}  
    
	/* avance d'un coup */
	void moveW(Coup& cp);
	void moveB(Coup& cp);
    
	/* recul d'un coup */
	void undoW(const Coup& cp);
	void undoB(const Coup& cp);

	/* "coup fantome" */
	void ghostMoveW(Coup& cp) { 
		if (cp.enPassant>=0) board[cp.enPassant]=0;
		cp.capture=board[cp.arr];
		board[cp.arr]=board[cp.dep];
		board[cp.dep]=0;
		return;
	}
	void ghostMoveB(Coup& cp) { 
		if (cp.enPassant>=0) board[cp.enPassant]=0;
		cp.capture=board[cp.arr];
		board[cp.arr]=board[cp.dep];
		board[cp.dep]=0;
		return;
	}
	void ghostUndoW(const Coup& cp) { 
		if (cp.enPassant>=0) board[cp.enPassant]=-1;
		board[cp.dep]=board[cp.arr];
		board[cp.arr]=cp.capture;
		return;
	}
	void ghostUndoB(const Coup& cp) { 
		if (cp.enPassant>=0) board[cp.enPassant]=1;
		board[cp.dep]=board[cp.arr];
		board[cp.arr]=cp.capture;
		return;
	}
	
	/* "coup fantome" sans prise en passant */
	void ghostMoveNEP_W(Coup& cp) { 
		cp.capture=board[cp.arr];
		board[cp.arr]=board[cp.dep];
		board[cp.dep]=0;
		return;
	}
	void ghostMoveNEP_B(Coup& cp) { 
		cp.capture=board[cp.arr];
		board[cp.arr]=board[cp.dep];
		board[cp.dep]=0;
		return;
	}
	void ghostUndoNEP_W(const Coup& cp) { 
		board[cp.dep]=board[cp.arr];
		board[cp.arr]=cp.capture;
		return;
	}
	void ghostUndoNEP_B(const Coup& cp) { 
		board[cp.dep]=board[cp.arr];
		board[cp.arr]=cp.capture;
		return;
	}
	
	/* "coup fantome" special roi */
	void ghostMoveKW(Coup& cp) { 
		cp.capture=board[cp.arr];
		board[cp.arr]=6;
		board[cp.dep]=0;
		roiB=cp.arr;
		return;
	}
	void ghostMoveKB(Coup& cp) { 
		cp.capture=board[cp.arr];
		board[cp.arr]=-6;
		board[cp.dep]=0;
		roiN=cp.arr;
		return;
	}
	void ghostUndoKW(const Coup& cp) { 
		board[cp.dep]=6;
		board[cp.arr]=cp.capture;
		roiB=cp.dep;
		return;
	}
	void ghostUndoKB(const Coup& cp) { 
		board[cp.dep]=-6;
		board[cp.arr]=cp.capture;
		roiN=cp.dep;
		return;
	} 

	/* calcul des cles du plateau pr hash */
	Clefs cle() const {
		unsigned int keyB=0;
		unsigned long long keyR=0;
		for (int i=63; i>=0; i--) {
			keyB+=aleaB[i]*board[i];
			keyR+=alea[i]*board[i];
		}
		return Clefs(keyB&(NB_MAX-1),keyR);
	}
	
	/* calcul de la signature du plateau pr bib */
	Signature codeBiblio() const {
		int clef=0;
		std::string position="                                                                ";
		for (int i=63; i>=0; i--) {
			position[i]=(char)(board[i]+64);
			clef+=aleaBib[i]*board[i];
		}
		return Signature(position,(abs(clef))%MAX_BIB);
	}

	/* compte le nombre de cases ~heuristiquement controlees par une piece. */
	int compteCasesW(int i) const;
	int compteCasesB(int i) const;
 
	/* valuation statique d'une position, pour une couleur donnee */
	float evalPosW() const; 
	float evalPosB() const; 
     
	/* destructeur */
	~Plateau() {} 
  
	/* emplacement actuel des rois blancs et noirs */
	int roiB,roiN;

	/* tabulation des operations modulo et DE (par 8) */
	int plMod8[64];
	int plDiv8[64];

	/* Stockage du plateau */
	int board[64];

private:
	
	/* nombres aleatoires pour hashages de Zobrist */
	unsigned long long alea[64];
	unsigned int aleaB[64];
	int aleaBib[64];
  
	/* valeur des pieces (modulables) */
	float VP,VC,VF,VT,VD;
  
	/* variable globale memorisant la valeur du jeu */
	float valW; /* valW-valB + si avantage blanc, - si avantage noir */
	float valB; 
  
};

/* Affichage du plateau */
extern std::ostream& operator<<(std::ostream& o, const Plateau& p);

#endif
